define( [ 'require',
    //'$App/mapping/uihandler',
    //'$App/mapping/datasources',
    '$App/mapping/entry',
    '$App/mapping/transformation',
    'lodash',
    //'$App/storage/api'
    ], function( require ){

    var //Storages = require( '$App/storage/api' ),
        _ = require( 'lodash' ),
        Entry = require( '$App/mapping/entry' ),
        Transformation = require( '$App/mapping/transformation' );
        //UIHandler = require( '$App/mapping/uihandler' ),
        //DataSources = require( '$App/mapping/datasources' );

	function MappingUnit(mappingUnit) {
        // Entry "from"
        this.from_name = mappingUnit.from_name;
        this.from_type = mappingUnit.from_type;

        // Entry "to"
        this.to_name = mappingUnit.to_name;
        this.to_type = mappingUnit.to_type;
        this.to_default = mappingUnit.to_default;

        // Mappings between participants
        this.mappings = mappingUnit.mappings;

        // Data buffers
        // will be initialized only with mapping process execution
        this.srcData = null;
        this.dstData = null;
    }

    var DATASOURCE_GROUP = ["SERVICE_REQUEST", "SERVICE_RESPONSE", "SERVICE_ERROR"],
        WEB_STORAGE_GROUP = ["SESSION_STORAGE", "LOCAL_STORAGE", "VARIABLE_STORAGE"];

    MappingUnit.prototype.process = function (source, target) {
        this.beforeProcessMappings(source, target);
        this.processMappings();
        /**
         * return results of processing
         */
        return this.dstData;
    };

    MappingUnit.prototype.beforeProcessMappings = function (source, target) {
        // JSON - *
        if (this.from_name && this.from_type) {
            this.srcData = _.deepClone(source);
        }

        // JSON - JSON
        if (this.to_name && this.to_type) {
            this.dstData = _.deepClone(target);
        }
        if (this.to_default && $.isEmptyObject(this.dstData)) {
            this.dstData = this.__applyDefaultValues(this.dstData, this.to_default);
        }

    };

    MappingUnit.prototype.processMappings = function () {
        for (var i = 0; i < this.mappings.length; i++) {
            var from = new Entry(this, this.mappings[i].source),
                to = new Entry(this, this.mappings[i].target),
                transformation = new Transformation(this.mappings[i].source_transformation,
                    this.mappings[i].target_transformation);

            // Source transformation only case
            if (!to.path) {
                this.srcData = MEP.merge(from.path, this.srcData, MEP.get(from.path, this.srcData), transformation);
                continue;
            }

            // Target transformation only case
            if (!from.path) {
                this.dstData = MEP.merge(to.path, this.dstData, MEP.get(to.path, this.dstData), transformation);
                continue;
            }

            // Check if mapping is malformed (dynamic indices count must match)
            var fromICount = from.path.match(/\[i\]/g);
            fromICount = fromICount ? fromICount.length : 0;
            var toICount = to.path.match(/\[i\]/g);
            toICount = toICount ? toICount.length : 0;

            if (this.from_type !== "UI" && this.to_type !== "UI" && fromICount !== toICount) {
                console.error("Bad mapping " + from.path + " > " + to.path);
            }

            // Perform data transfer
            to.setValue(from.getValue(), transformation);
        }
    };

    MappingUnit.prototype.__applyDefaultValues = function (data, defaultValues) {
        if (defaultValues === undefined) return data;
        if (!_.isTrueObject(defaultValues)) throw new Error("Default values argument must be an object");

        var result;

        for (var prop in defaultValues) {
            if (defaultValues.hasOwnProperty(prop)) {
                result = result || {};

                if (_.isTrueObject(defaultValues[prop])) {
                    result[prop] = MappingUnit.prototype.__applyDefaultValues(
                            data && _.isTrueObject(data[prop]) ? data[prop] : {}, defaultValues[prop]);
                } else {
                    result[prop] = data && data[prop] !== undefined ? data[prop] : defaultValues[prop];
                }
            }
        }

        return result !== undefined ? result : data;
    };

    MappingUnit.prototype.DATASOURCE_GROUP = DATASOURCE_GROUP;
    MappingUnit.prototype.WEB_STORAGE_GROUP = WEB_STORAGE_GROUP;

    return MappingUnit;
});
