define( [ 'require', 'lodash' ], function( require, _ ){

    /**
     * Runs user JS-code for data
     */

    function Transformation(sourceFunc, targetFunc) {
        this.sourceFunc = sourceFunc;
        this.targetFunc = targetFunc;
    }

    Transformation.prototype.transform = function (value, element) {
        var buf,
            result = value,
            args = [];
        if (element) args[1] = element;

        if (_.isFunction(this.sourceFunc)) {
            args[0] = _.deepClone(result);

            try {
                buf = this.sourceFunc.apply(this, args);
            } catch (e) {
                console.error("User transformation function threw an exception. Reason: ", e.message);
            }

            if (!_.isUndefined(buf)) {
                result = buf;
                buf = undefined;
            }
        }

        if (_.isFunction(this.targetFunc)) {
            args[0] = _.deepClone(result);

            try {
                buf = this.targetFunc.apply(this, args);
            } catch (e) {
                console.error("User transformation function exception. Reason: ", e.message);
            }

            if (!_.isUndefined(buf)) {
                result = buf;
                buf = undefined;
            }
        }

        return result;
    };

    return Transformation;
});
